library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.vga_ref_pack.all;

entity vga_ref is
	Port ( clk : in  std_logic;
		    rst : in  std_logic;
		    hs  : out std_logic;
		    vs  : out std_logic;
		    rgb_out : out std_logic_vector (11 downto 0)
		  );
end vga_ref;

architecture Behavioral of vga_ref is

component clk_wiz_0 is
 port (
  clk_in1   : in std_logic;
  clk_out1  : out std_logic;
  reset     : in std_logic;
  locked    : out std_logic
 );
 end component;

	
	component vga_controller_640_60 is
      port ( rst       : in  std_logic; 
             pixel_clk : in  std_logic; 
             HS        : out std_logic; 
             VS        : out std_logic; 
             blank     : out std_logic; 
             hcount    : out std_logic_vector(10 downto 0); 
             vcount    : out std_logic_vector(10 downto 0)
			  );
   end component;
	
	component rom_welcome is
		port ( addra      : in  std_logic_vector(15 downto 0);
			   clka       : in  std_logic;
			   douta	  : out std_logic_vector(2 downto 0)
		 	  );
   end component;
	
	-- General signals
	signal clk_sys	: std_logic; 
	signal clk_locked, rst_sys : std_logic;
	
	-- VGA module
	signal blank : std_logic;
	signal hcount,vcount : std_logic_vector(10 downto 0);
	
	-- Picture ROM
	signal rom_addr : std_logic_vector(15 downto 0);
	signal rom_dout : std_logic_vector(2 downto 0);
	
	signal r_out    : std_logic_vector(3 downto 0);
	signal g_out    : std_logic_vector(3 downto 0);
	signal b_out    : std_logic_vector(3 downto 0);
	
	signal rgb      : std_logic_vector(2 downto 0);
	
begin

	rst_sys <= rst or (not clk_locked);		-- Release system reset when clock is stable
	
	
	-- Replicate the r g and b signals for nexys 4 board
	rgb_out(11 downto 8) <= r_out;
	rgb_out(7 downto 4)  <= g_out;
	rgb_out(3 downto 0)  <= b_out;
	
	r_out <= rgb(2) & rgb(2) & rgb (2) & rgb(2);
	g_out <= rgb(1) & rgb(1) & rgb (1) & rgb(1);
	b_out <= rgb(0) & rgb(0) & rgb (0) & rgb(0);
	
	
	Inst_clock_gen:
	clk_wiz_0
	port map (   clk_in1  	=> clk,
				 clk_out1 	=> clk_sys,			-- Don't touch! active high reset
				 reset    	=> rst,		-- Divided 50MHz input clock
				 locked     => clk_locked
				);

	vgactrl640_60:
	vga_controller_640_60
   port map ( pixel_clk => clk_sys,
              rst			=> rst_sys,
              blank		=> blank,
              hcount		=> hcount,
              hs			=> hs,
              vcount		=> vcount,
              vs			=> vs
				);
				
	picture_rom:
	rom_welcome
	port map ( clka  => clk_sys,
	           addra => rom_addr,
	           douta => rom_dout);
	
	picture_display:
	process (clk_sys, rst_sys)
	begin
		if rst_sys = RESET_SYSTEM then
			rom_addr <= (others => '0');
		elsif clk_sys = '1' and clk_sys'event then
			if blank = '1' then
				rgb <= (others => '0');		-- Have to be zeros during the blank period
			else
				rgb <= COLOR_WHITE;			-- Background color
					
				if vcount >= MESSAGE_START_V and vcount <= MESSAGE_START_V + MESSAGE_HEIGHT then
					if hcount >= MESSAGE_START_H and hcount <= MESSAGE_START_H + MESSAGE_WIDTH - 1 then		-- "-1" is due to the registered "hcount" output
						rgb <= rom_dout;
						
						if (rom_addr = ROM_DEPTH - 1 or vcount = MESSAGE_START_V) then		-- ROM address is counted from 0, therefore "ROM_DEPTH - 1"
							rom_addr <= (others => '0');
						else
							rom_addr <= rom_addr + 1;
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;
end Behavioral;
